@@ -1,8 +1,16 @@
+/*
+ * Copyright (C) 1996-2018 The Squid Software Foundation and contributors
+ *
+ * Squid software is distributed under GPLv2+ license and includes
+ * contributions from numerous individuals and organizations.
+ * Please see the COPYING and CONTRIBUTORS files for details.
+ */
+
 #ifndef _INC_INET_PTON_H
 #define _INC_INET_PTON_H
 
 /* Use the system provided version where possible */
-#if !HAVE_INET_PTON
+#if !HAVE_DECL_INET_PTON
 
 /* int
 * inet_pton(af, src, dst)
@@ -18,5 +26,6 @@
 SQUIDCEXTERN int xinet_pton(int af, const char *src, void *dst);
 #define inet_pton xinet_pton
 
-#endif
+#endif /* HAVE_DECL_INET_PTON */
 #endif /* _INC_INET_NTOP_H */
+
