@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2016 The Squid Software Foundation and contributors
+ * Copyright (C) 1996-2018 The Squid Software Foundation and contributors
  *
  * Squid software is distributed under GPLv2+ license and includes
  * contributions from numerous individuals and organizations.
@@ -919,8 +919,8 @@ get_memberof(struct main_args *margs, char *user, char *domain, char *group)
     /*
      * Initialise ldap
      */
-    ldap_debug = 127 /* LDAP_DEBUG_TRACE */ ;
-    ldap_debug = -1 /* LDAP_DEBUG_ANY */ ;
+//    ldap_debug = 127 /* LDAP_DEBUG_TRACE */ ;
+//    ldap_debug = -1 /* LDAP_DEBUG_ANY */ ;
     ldap_debug = 0;
     (void) ldap_set_option(NULL, LDAP_OPT_DEBUG_LEVEL, &ldap_debug);
 #endif
