@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2014 The Squid Software Foundation and contributors
+ * Copyright (C) 1996-2018 The Squid Software Foundation and contributors
  *
  * Squid software is distributed under GPLv2+ license and includes
  * contributions from numerous individuals and organizations.
@@ -41,19 +41,32 @@ DelaySpec::dump (StoreEntry *entry) const
 void
 DelaySpec::parse()
 {
-    int r;
-    char *token;
-    token = strtok(NULL, "/");
-
+    // get the token.
+    char *token = ConfigParser::NextToken();
     if (token == NULL)
         self_destruct();
 
-    if (sscanf(token, "%d", &r) != 1)
+    // no-limit value
+    if (strcmp(token, "none") == 0 || token[0] == '-') {
+        restore_bps = -1;
+        max_bytes = -1;
+        return;
+    }
+
+    // parse the first digits into restore_bps
+    const char *p = NULL;
+    if (!StringToInt(token, restore_bps, &p, 10) || *p != '/') {
+        debugs(77, DBG_CRITICAL, "ERROR: invalid delay rate '" << token << "'. Expecting restore/max or 'none'.");
         self_destruct();
+    }
+    p++; // increment past the '/'
 
-    restore_bps = r;
-
-    max_bytes = GetInteger64();
+    // parse the rest into max_bytes
+    if (!StringToInt64(p, max_bytes, NULL, 10)) {
+        debugs(77, DBG_CRITICAL, "ERROR: restore rate in '" << token << "' is not a number.");
+        self_destruct();
+    }
 }
 
 #endif
+
