@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2015 The Squid Software Foundation and contributors
+ * Copyright (C) 1996-2018 The Squid Software Foundation and contributors
  *
  * Squid software is distributed under GPLv2+ license and includes
  * contributions from numerous individuals and organizations.
@@ -31,8 +31,9 @@ Auth::User::User(Auth::Config *aConfig, const char *aRequestRealm) :
     notes(),
     credentials_state(Auth::Unchecked),
     username_(NULL),
-    requestRealm_(aRequestRealm)
+    userKey_(NULL)
 {
+    requestRealm_ = aRequestRealm ? xstrdup(aRequestRealm) : NULL;
     proxy_match_cache.head = proxy_match_cache.tail = NULL;
     ip_list.head = ip_list.tail = NULL;
     debugs(29, 5, HERE << "Initialised auth_user '" << this << "'.");
@@ -134,6 +135,10 @@ Auth::User::~User()
 
     if (username_)
         xfree((char*)username_);
+    if (requestRealm_)
+        xfree((char*)requestRealm_);
+    if (userKey_)
+        xfree((char*)userKey_);
 
     /* prevent accidental reuse */
     auth_type = Auth::AUTH_UNKNOWN;
@@ -279,7 +284,7 @@ Auth::User::addIp(Ip::Address ipaddr)
             /* This ip has already been seen. */
             found = 1;
             /* update IP ttl */
-            ipdata->ip_expiretime = squid_curtime;
+            ipdata->ip_expiretime = squid_curtime + ::Config.authenticateIpTTL;
         } else if (ipdata->ip_expiretime <= squid_curtime) {
             /* This IP has expired - remove from the seen list */
             dlinkDelete(&ipdata->node, &ip_list);
@@ -365,14 +370,17 @@ Auth::User::UsernameCacheStats(StoreEntry *output)
 void
 Auth::User::username(char const *aString)
 {
+    SBuf key;
+
     if (aString) {
         assert(!username_);
         username_ = xstrdup(aString);
-        // NP: param #2 is working around a c_str() data-copy performance regression
-        userKey_ = BuildUserKey(username_, (!requestRealm_.isEmpty() ? requestRealm_.c_str() : NULL));
+        key = BuildUserKey(username_, requestRealm_);
+        // XXX; performance regression. c_str() reallocates, then xstrdup() reallocates
+        userKey_ = xstrdup(key.c_str());
     } else {
         safe_free(username_);
-        userKey_.clear();
+        safe_free(userKey_)
     }
 }
 
