@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2015 The Squid Software Foundation and contributors
+ * Copyright (C) 1996-2018 The Squid Software Foundation and contributors
  *
  * Squid software is distributed under GPLv2+ license and includes
  * contributions from numerous individuals and organizations.
@@ -124,9 +124,8 @@ Auth::UserRequest::getDenyMessage()
 char const *
 Auth::UserRequest::denyMessage(char const * const default_message)
 {
-    if (this == NULL || getDenyMessage() == NULL) {
+    if (getDenyMessage() == NULL)
         return default_message;
-    }
 
     return getDenyMessage();
 }
