@@ -1,10 +1,18 @@
-#define SQUID_UNIT_TEST 1
+/*
+ * Copyright (C) 1996-2018 The Squid Software Foundation and contributors
+ *
+ * Squid software is distributed under GPLv2+ license and includes
+ * contributions from numerous individuals and organizations.
+ * Please see the COPYING and CONTRIBUTORS files for details.
+ */
+
 #include "squid.h"
-#include "testConfigParser.h"
-#include "SquidString.h"
-#include "Mem.h"
-#include "event.h"
 #include "ConfigParser.h"
+#include "event.h"
+#include "Mem.h"
+#include "SquidString.h"
+#include "testConfigParser.h"
+#include "unitTestMain.h"
 
 CPPUNIT_TEST_SUITE_REGISTRATION( testConfigParser);
 
@@ -54,24 +62,24 @@ bool testConfigParser::doParseQuotedTest(const char *s, const char *expectInterp
 void testConfigParser::testParseQuoted()
 {
     // SingleToken
-    CPPUNIT_ASSERT(doParseQuotedTest("SingleToken", "SingleToken"));
+    CPPUNIT_ASSERT_EQUAL(true, doParseQuotedTest("SingleToken", "SingleToken"));
 
     // This is a quoted "string" by me
-    CPPUNIT_ASSERT(doParseQuotedTest("\"This is a quoted \\\"string\\\" by me\"",
-                                     "This is a quoted \"string\" by me"));
+    CPPUNIT_ASSERT_EQUAL(true, doParseQuotedTest("\"This is a quoted \\\"string\\\" by me\"",
+                         "This is a quoted \"string\" by me"));
 
     // escape sequence test: \\"\"\\"
-    CPPUNIT_ASSERT(doParseQuotedTest("\"escape sequence test: \\\\\\\\\\\"\\\\\\\"\\\\\\\\\\\"\"",
-                                     "escape sequence test: \\\\\"\\\"\\\\\""));
+    CPPUNIT_ASSERT_EQUAL(true, doParseQuotedTest("\"escape sequence test: \\\\\\\\\\\"\\\\\\\"\\\\\\\\\\\"\"",
+                         "escape sequence test: \\\\\"\\\"\\\\\""));
 
     // \beginning and end test"
-    CPPUNIT_ASSERT(doParseQuotedTest("\"\\\\beginning and end test\\\"\"",
-                                     "\\beginning and end test\""));
+    CPPUNIT_ASSERT_EQUAL(true, doParseQuotedTest("\"\\\\beginning and end test\\\"\"",
+                         "\\beginning and end test\""));
 
     // "
-    CPPUNIT_ASSERT(doParseQuotedTest("\"\\\"\"", "\""));
+    CPPUNIT_ASSERT_EQUAL(true, doParseQuotedTest("\"\\\"\"", "\""));
 
     /* \ */
-    CPPUNIT_ASSERT(doParseQuotedTest("\"\\\\\"", "\\"));
+    CPPUNIT_ASSERT_EQUAL(true, doParseQuotedTest("\"\\\\\"", "\\"));
 }
 
